////////////////////////////////////////////////
//
//     -- Mikael Beckius 2008 --
//
////////////////////////////////////////////////

#include <EGL\egl.h>
#include <GLES2\gl2.h>
#include <GLES2\gl2ext.h>
#include "fighterRGBA.h"
#include "vertexShader.h"
#include "pixelShader.h"

#include <string.h>
#include <stdlib.h>

#define GL_BGRA  0x80E1 // Not implemented for OpenGL ES 2.0. Only OpenGL ES 1.1

static GLfloat vertices[] ={-40,40, -40,-40, 40,40, 40,-40}; 
static GLfloat texCoords[] = {0.0,1.0, 0.0,0.0, 1.0,1.0, 1.0,0.0};

float* mbOrthoMatrix(float m[16],float l,float r,float b,float t,float n,float f){
	m[0] = 2/(r-l);
	m[1] = 0;
	m[2] = 0;
	m[3] = 0;

	m[4] = 0;
	m[5] = 2/(t-b);
	m[6] = 0;
	m[7] = 0;

	m[8] = 0;
	m[9] = 0;
	m[10] = -2/(f-n);
	m[11] = 0;

	m[12] = (r+l)/(r-l);
	m[13] = (t+b)/(t-b);
	m[14] = (f+n)/(f-n);
	m[15] = 1;
	return m;
}

unsigned int createShader(char* shaderStr,unsigned int m_program,unsigned int shaderType){
	unsigned int m_shader;
	int	length, maxLength;
	int compiled  = GL_FALSE;
	char *infoLog;
	int sourceSize;

	sourceSize = (int)strlen(shaderStr);
	m_shader = glCreateShader(shaderType);

	glShaderSource(m_shader,1,(const char**)&shaderStr,&sourceSize); // Specify source code
	glCompileShader(m_shader); // Compile

	glGetShaderiv(m_shader,GL_COMPILE_STATUS,&compiled);
	glGetShaderiv(m_shader,GL_INFO_LOG_LENGTH,&maxLength);
	infoLog = (char*) malloc(maxLength * sizeof(char));
	glGetShaderInfoLog(m_shader,maxLength,&length,infoLog);

	// printf("%s\n",shaderStr);
	if(compiled){
		glAttachShader(m_program,m_shader); // Add
	}
	else{
		printf("%s\n",infoLog);
		printf("Compile failed\n");
	}
	free(infoLog);
	infoLog=0;
	return 0;
}

void linkShader(unsigned int m_program,unsigned int m_shader){
	int	length, maxLength;
	int linked = GL_FALSE;
	char *infoLog;

	// Optionally add more shaders to the program and link the program
	glLinkProgram(m_program);
	glGetProgramiv(m_program,GL_LINK_STATUS,&linked);

	glGetProgramiv(m_program,GL_INFO_LOG_LENGTH,&maxLength);
	infoLog = (char*) malloc(maxLength*sizeof(char));
	glGetProgramInfoLog(m_program,maxLength,&length,infoLog);

	// If all went well, make the program part of the current state
	if (!linked) {
		printf("%s\n", infoLog);
		printf("Link failed\n");
	}
	free(infoLog);
}

int main(int argc, char** argv){
	int frame = 0;
	TTime beginTime;
	TTime endTime;
	float modelViewProj[16]; 

	EGLDisplay dpy;
	EGLSurface surface;
	EGLContext context;
	EGLConfig configs[2];
	EGLint config_count;
	EGLint major, minor;
	EGLint context_attribs[] = {EGL_CONTEXT_CLIENT_VERSION, 2, EGL_NONE};
	EGLint cfg_attribs[] = {EGL_BUFFER_SIZE, EGL_DONT_CARE,
		// EGL_SURFACE_TYPE,EGL_WINDOW_BIT,
		EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
		EGL_RED_SIZE,       8,
		EGL_GREEN_SIZE,     8,
		EGL_BLUE_SIZE,      8,
		EGL_ALPHA_SIZE,     8,
		// EGL_DEPTH_SIZE,			24,
		EGL_NONE};

	dpy = eglGetDisplay (EGL_DEFAULT_DISPLAY);
	if(!eglInitialize(dpy, &major, &minor)){
		printf("eglInitialize failed");
	}
	if(!eglGetConfigs (dpy, configs, 2, &config_count)){
		printf("eglGetConfigs failed");
	}
	if(!eglChooseConfig (dpy, cfg_attribs, configs, 2, &config_count)){
		printf("eglChooseConfig failed");
	}
	surface = eglCreateWindowSurface (dpy, configs[0], 0, 0);
	if(!surface){
		printf("eglCreateWindowSurface failed");
	}
	if(!eglBindAPI(EGL_OPENGL_ES_API)){
		printf("eglBindAPI failed");
	}
	context = eglCreateContext (dpy, configs[0], EGL_NO_CONTEXT, context_attribs);
	if(!context){
		printf("eglCreateContext failed");
	}

	if(!eglMakeCurrent(dpy, surface, surface, context)){
		printf("eglMakeCurrent failed");
	}

	unsigned int m_program = glCreateProgram();
	unsigned int m_vertexShader = createShader(vertexShader,m_program,GL_VERTEX_SHADER);
	unsigned int m_fragmentShader = createShader(pixelShader,m_program,GL_FRAGMENT_SHADER);

	glBindAttribLocation(m_program, 0, "position");
	glBindAttribLocation(m_program, 1, "inTexCoord");

	linkShader(m_program,m_vertexShader);
	linkShader(m_program,m_fragmentShader);

	glUseProgram(m_program);

	glEnableVertexAttribArray (0);
	glEnableVertexAttribArray (1);

	glVertexAttribPointer(glGetAttribLocation(m_program,"position"), 2, GL_FLOAT, 0, 0, vertices);
	glVertexAttribPointer(glGetAttribLocation(m_program,"inTexCoord"), 2, GL_FLOAT, 0, 0, texCoords);

	mbOrthoMatrix(modelViewProj,-24,24,-32,32,-100.0,100.0);

	glUniformMatrix4fv(glGetUniformLocation(m_program,"mWorldViewProj"),1,GL_FALSE,modelViewProj);
	glUniform1i(glGetUniformLocation(m_program,"mTexture"),1);
	glClearColor(0.0,0.0,1.0,0.0);
	glViewport(0,0,240,320);

	unsigned int id;
	glGenTextures(1,&id);
	glBindTexture(GL_TEXTURE_2D,id);
	glPixelStorei(GL_UNPACK_ALIGNMENT,1);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 256, 256, 0, GL_RGBA, GL_UNSIGNED_BYTE, imageFighterRGBA);

	GLenum e;
	do{
		e=glGetError();
		printf("main glGetError %u\n",e);
	}while(e!=GL_NO_ERROR);

	while(frame++<10000){
		glClear(GL_COLOR_BUFFER_BIT);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_BGRA, 256, 256, 0, GL_BGRA, GL_UNSIGNED_BYTE, imageFighterRGBA);
		glDrawArrays(GL_TRIANGLE_STRIP,0,4);
		eglSwapBuffers (dpy, surface);

	}
	eglMakeCurrent(dpy, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
	eglDestroyContext(dpy, context);
	eglDestroySurface(dpy, surface);
	eglTerminate(dpy);
	return 0;
}
